# %%
import os
import time
from types import NoneType

import mat73
import numpy as np

import pymatio as pm

# %%
os.system("clear")

def is_iterable(a):
    return isinstance(a, (list, tuple, set, frozenset, np.ndarray))

def compare_maybe_array(a, b):
    if {type(a), type(b)} == {np.ndarray, NoneType}:
        if a is None: return b.size == 0
        if b is None: return a.size == 0
        return False
    
    if is_iterable(a) and not isinstance(b, np.ndarray) and len(a) == 1:
        return a[0] == b
    if is_iterable(b) and not isinstance(a, np.ndarray) and len(b) == 1:
        return a == b[0]

    if isinstance(a, np.ndarray) and isinstance(b, np.ndarray):
        # print('cmp', a, b)
        try:
            return np.array_equal(a.reshape(-1), b.reshape(-1))
        except Exception as e:
            if a.size != b.size or a.dtype != b.dtype:
                # print(f"形状不一致: {a.shape} - {b.shape}")
                return False
            else:
                for i in range(a.size):
                    try:
                        eq = compare_mats(
                            a.reshape(-1)[i], b.reshape(-1)[i]
                        )
                        if not eq: return False
                    except:
                        print('cmp error', i, a, b)

    else:
        return a == b

def compare_mats(mat1, mat2, path=""):
    if isinstance(mat1, dict) and isinstance(mat2, dict):
        for key in set(mat1.keys()) | set(mat2.keys()):
            if key not in mat1:
                print(f"字段 '{path}{key}' 在 mat_from_pm 中不存在")
            elif key not in mat2:
                print(f"字段 '{path}{key}' 在 mat_from_mat73 中不存在")
            else:
                compare_mats(mat1[key], mat2[key], f"{path}{key}.")
    elif is_iterable(mat1) and is_iterable(mat2):
        if len(mat1) != len(mat2):
            print(f"列表长度不一致: {path[:-1]}")
        else:
            for i, (item1, item2) in enumerate(zip(mat1, mat2)):
                compare_mats(item1, item2, f"{path}[{i}].")
    elif any(type(m) in (float, int, bool, np.bool, np.float64, np.uint8) for m in (mat1, mat2)):
        return mat1 == mat2
    elif mat1 is None and isinstance(mat2, np.ndarray) and mat2.size == 0:
        return True
    elif mat2 is None and isinstance(mat1, np.ndarray) and mat1.size == 0:
        return True
    elif not compare_maybe_array(mat1, mat2):
        print(f"值不一致: {path[:-1]} `{mat1}` <-> `{mat2}`")
        
        
p = '/home/myuan/C052-inj/C052-217-P0.ntp'
# p = '/mnt/90-connectome/test_data.mat'
# p = '/mnt/90-connectome/test_data_73.mat'
t0 = time.time()
mat_from_pm = pm.loadmat(p, simplify_cells=True)
t1 = time.time()

try:
    mat_from_mat73 = mat73.loadmat(p)
    # mat_from_mat73 = hdf5storage.loadmat(p, simplify_cells=True, chars_as_strings=True)

except Exception as e:
    import scipy.io as scio
    mat_from_mat73 = (scio.loadmat(p, simplify_cells=True, chars_as_strings=True))
t2 = time.time()

print('', flush=True)

print(f'pm.loadmat: {t1-t0:.4f}s')
print(f'mat73.loadmat: {t2-t1:.4f}s')

compare_mats(mat_from_pm, mat_from_mat73)
# print(mat_from_pm['testStruct']['intArray'])
# print('---'*30)
# print(mat_from_pm['cf']['textinfo'])
# print('---'*30)
# print(mat_from_mat73['cf']['textinfo'])
# print(mat_from_pm, mat_from_mat73, sep='\n\n')
# %%
# %%
def show_arr_and_flags(arr):
    # print(arr)
    print(arr.flags)
    print(arr.strides)
    print(arr.shape)
    print('-'*40)

# show_arr_and_flags(mat_from_pm['testStruct']['cellArrayMul'])
# show_arr_and_flags(mat_from_mat73['testStruct']['cellArrayMul'])
# %%



import importlib
import importlib.util
import sys
from pathlib import Path
import sysconfig

EXT_SUFFIX = sysconfig.get_config_var("EXT_SUFFIX")

candidate_dlls = [
    Path(__file__).parent / f'libpymatio{EXT_SUFFIX}',
    Path(__file__).parent.parent / 'build' / f'libpymatio{EXT_SUFFIX}',
    # *list(Path(__file__).parent.parent.glob(f'libpymatio{EXT_SUFFIX}'))
]
candidate_dlls = filter(lambda x: x.exists(), candidate_dlls)
candidate_dlls = sorted(candidate_dlls, key=lambda x: x.stat().st_size, reverse=True)

for target_dll in candidate_dlls:
    if not target_dll.exists():
        continue
    try:
        spec = importlib.util.spec_from_file_location("libpymatio", target_dll)
        libpymatio = importlib.util.module_from_spec(spec)
        sys.modules["libpymatio"] = libpymatio
        spec.loader.exec_module(libpymatio)
        break
    except Exception as e:
        raise
else:
    raise RuntimeError(f"Failed to load pymatio. Candidates: {candidate_dlls}")

# from libpymatio import get_library_version
from libpymatio import *



import os
import shutil
import subprocess
import sys
import sysconfig
from pathlib import Path
import logging

from pybind11.commands import get_include as get_pybind11_include
from setuptools import Extension
from setuptools.command.build_ext import build_ext


class XmakeBuildExt(build_ext):
    def run(self):
        self.xmake_build()
        self.copy_output_file()

    def xmake_build(self):
        platform = sysconfig.get_platform()
        xmake_archs = {
            'win-amd64': 'x64',
            'win32': 'x86',
            'linux-x86_64': 'x86_64',
            'linux-i686': 'i386',
            'darwin-x86_64': 'x86_64',
            'darwin-arm64': 'arm64',
        }
        curr_arch = xmake_archs.get(platform)
        if curr_arch is None:
            raise Exception(f'Unsupported platform: {platform}, allowed: {xmake_archs}')
        logging.debug(f"{curr_arch=} {platform=}")
        python_include = sysconfig.get_path('include')
        if os.name != 'nt':
            python_lib = sysconfig.get_config_var('LIBDIR')
        else:
            python_lib = Path(python_include).parent.joinpath('libs').as_posix()
        python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
        python_bin = sysconfig.get_path("scripts")
        python_site_packages = sysconfig.get_path("purelib")
        pybind11_include = get_pybind11_include()
        

        # print(f'{python_include=} {python_lib=} {python_version=} {python_bin=} {python_site_packages=}')
        # print(list(Path(python_bin).parent.glob("*")))
        # print(Path(python_bin).is_symlink())
        # os.system(f"ls -l {python_bin}")
        # exit(1)

        # 检查xmake是否存在
        if not shutil.which("xmake"):
            raise EnvironmentError(
                f"xmake is not installed or not found in PATH. \nTo install xmake, please refer to https://xmake.io/#/guide/installation\n"
                f"PATH: {os.environ['PATH']}"
            )
        sep = ";" if os.name == "nt" else ":"
        env = {
            **os.environ,
            "XMAKE_PYTHON_INCLUDE": python_include or '', 
            "XMAKE_PYTHON_LIB": python_lib or '',
            "XMAKE_PYTHON_VERSION": python_version or '',
            "XMAKE_PYTHON_SITE_PACKAGES": python_site_packages or '',
            "XMAKE_PYTHON_BIN": python_bin or '',
            "PATH": f"{python_bin}{sep}{python_site_packages}{sep}{os.environ['PATH']}",
            "XMAKE_PYBIND11_INCLUDE": pybind11_include,
        }
        if os.name != 'nt':
            subprocess.run(["env"], env=env)
        if os.environ.get("SLEEP"):
            print("sleep 100000")
            os.system("sleep 100000")

        # subprocess.run(["xmake", "config", "-c", "-a", curr_arch, "-v", "-D", '-y', f"--includedirs={python_include}", f"--linkdirs={python_lib}"])
        config_cmds = ["xmake", "config", "-c", "-a", curr_arch, "-v", "-D", '-y']
        if os.getenv("XMAKE_DEBUG"):
            print("debug mode")
            config_cmds.extend(["-m", "debug"])
        if not os.getenv("XMAKE_NO_RUN_CONFIG"):
            subprocess.run(config_cmds, env=env)

        p = subprocess.run(["xmake", "build", '-y', '-v', '-D'], env=env)
        if p.returncode != 0:
            raise Exception(f"xmake build failed: {p.returncode}")


    def copy_output_file(self):
        ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
        built_files = list(Path('build').glob(f'**/libpymatio{ext_suffix}'))
        if not built_files:
            raise FileNotFoundError(f"No libpymatio file found after xmake build @ {Path('build').absolute()}")
        built_file = str(built_files[0])
        logging.info(f"Found built library: {built_file} @ {Path('.').absolute()}")
        
        self.mkpath(os.path.join(self.build_lib, 'pymatio'))
        self.copy_file(built_file, os.path.join(self.build_lib, 'pymatio'))
        self.copy_extensions_to_source()


def build(setup_kwargs):
    print(setup_kwargs)
    setup_kwargs.update({
        'ext_modules': [Extension('pymatio.libpymatio', sources=[])],
        'cmdclass': {'build_ext': XmakeBuildExt},
        'package_data': {
            'pymatio': [f'libpymatio{sysconfig.get_config_var("EXT_SUFFIX")}'],
        },
    })

    print(setup_kwargs)
